function [x,info] = bgs(funObj,lambda,groups,weights,opts)
% BGS   Block Gauss-Seidel solver
%
%   [X,INFO] = BGS(FUNOBJ,LAMBDA,GROUPS,WEIGHTS,OPTS) solves
%
%       Minimize  funObj(x) + lambda * ||x||_WG,1
%   or
%       Minimize  funObj(x) + lambda * sum(w_i ||x_Gi||_2)
%
%   With weighted group norm depending on GROUPS and WEIGHTS.
%   The GROUPS vector indicates for each entry of variable X
%   which group (number) it belongs to. The length of WEIGHTS 
%   should correspond to the number of groups used. Group
%   numbering should be done from 1 to the number of groups.
%
%   The following fields of the OPTS options structure are
%   recognized:
%
%   bgstol	Tolerance on gradient (converged once all below)
%   maxiter     Maximum number of block iterations
%   maxfunc     Maximum number of function evaluations
%   verbosity   0 = no output, 1 = all output

if nargin < 5, opts = struct(); end;

if ~isfield(opts,'bgstol'   ), opts.bgstol    = 1e-3;            end;
if ~isfield(opts,'maxiter'  ), opts.maxiter   = 1000;            end;
if ~isfield(opts,'maxfunc'  ), opts.maxfunc   = 10*opts.maxiter; end;
if ~isfield(opts,'verbosity'), opts.verbosity = 1;               end;

% Maximum value of gradient for non-zero blocks
tol     = opts.bgstol;

% Set exit codes
EXIT_ITERATIONS = 1;
EXIT_OPTIMAL    = 2;
EXIT_FUNCTIONS  = 3;

% Set log format
logB = ' %5i  %13.7e  %13.7e  %4d  %4d';
logH = ' %5s  %13s  %13s  %10s  %6s\n';
logL = {'Iter','Objective','gNormMax','funeval','cycle'};
n    = strread(sprintf(strrep(strrep(logH,'%',' '),'s',' ')),'%d');
logD = arrayfun(@(l)(repmat('-',1,l)),n','UniformOutput',false);

if opts.verbosity, fprintf(logH,logL{:}); end;
if opts.verbosity, fprintf(logH,logD{:}); end

% Get group information
groupIdx = unique(groups);
nGroups  = length(groupIdx);
n        = length(groups);

% Initialize 
x        = zeros(n,1);
[f,g]    = funObj(x);
iter     = 0;
funeval  = 1;
block    = 1;  % Current block
stat     = 0;
cycle    = 0;  % Current cycle index
fHist    = zeros(1,opts.maxiter);
normHist = zeros(1,opts.maxiter);
evalHist = zeros(1,opts.maxiter);

while (stat == 0)
  
  % Get block information
  k        = groupIdx(block);
  idx      = (groups == k);
  
  % Initialize function evaluations for this iteration
  funeval0 = 0;
     
  % Compute gradient with local solution set to zero
  if (norm(x(idx)) ~= 0)
     xbar      = x;
     xbar(idx) = 0;
     [f,g]     = funObj(xbar);
     funeval0  = funeval0 + 1;
  else
     % Values of f and g are still up to date
  end

  if norm(g(idx)) < lambda * weights(k)
     % Set the local solution to zero
     x(idx) = zeros(sum(idx),1);
  else
     % Determine initial point
     if norm(x(idx)) > 1e-5
        xinit = x(idx);
        % The initial function evaluation of the local minimization
        % problem could have been avoided by reusing the existing
        % information. Take this into account when determining the
        % number of function evaluations.
        funeval0 = funeval0 - 1;
     else
        xinit = ones(sum(idx),1);
     end
        
     % Optimize local problem
     localFun = @(xgroup) groupObjective(funObj,lambda*weights(k),idx,x,xgroup);
     options  = optimset('Display','off');
     options.GradObj    = 'on';
     options.LargeScale = 'off';
     [xgroup,f,exitflag,output] = minFunc(localFun,xinit,options);
     
     % Keep track of number of function evaluations
     funeval0 = funeval0 + output.funcCount;
     
     % Update the local solution
     x(idx) = xgroup;
     
     % Evaluate the function
     [f,g]    = funObj(x);
     funeval0 = funeval0 + 1;
  end

  
  % Compute the group one-norm values and add to objective
  groupNorms = sqrt(accumarray(groups,x.^2));
  gradNorms  = sqrt(accumarray(groups,g.^2));
  objective  = f + lambda * sum(weights.*groupNorms);
  
  % Update number of function evaluations and history
  funeval          = funeval + funeval0;
  fHist(iter+1)    = f;
  normHist(iter+1) = sum(weights.*groupNorms);
  evalHist(iter+1) = funeval;

  % --------------------------------------------------------------------
  % Check stopping criteria
  % --------------------------------------------------------------------
  converged = 1;

  % Check if gradient is small for all zero blocks
  idx = (groupNorms == 0);
  if any(gradNorms(idx) >= lambda * weights(idx))
     converged = 0;
  end
  
  % Check if gradient is small for all non-zero blocks
  idx = find(groupNorms ~= 0); gradMax = 0;
  for i = idx'
     gidx = (groups == i);
     grad = g(gidx) + lambda * weights(i) * x(gidx) / norm(x(gidx));
     grad = norm(grad,inf);
     gradMax = max(gradMax,grad);
     if grad > tol
        converged = 0;
     end
  end

  % Determine final status
  if iter >= opts.maxiter, stat = EXIT_ITERATIONS; end;
  if funeval >= opts.maxfunc, stat = EXIT_FUNCTIONS; end;
  if converged, stat = EXIT_OPTIMAL; end;
 

  % --------------------------------------------------------------------
  % Display progress
  % --------------------------------------------------------------------
  if opts.verbosity, fprintf(logB,iter,objective,gradMax,funeval0,funeval); end;
  if (block == 1)
    cycle = cycle + 1;
    if opts.verbosity, fprintf('  %6d',cycle); end;
    
  end
  if opts.verbosity, fprintf('\n'); end;

  
  % Update iteration counter and block number
  iter    = iter + 1;
  block   = mod(block,nGroups) + 1;
end

% Update information
info = struct();
info.iter     = iter;
info.stat     = stat;
info.fHist    = fHist(1:iter);
info.normHist = normHist(1:iter);
info.evalHist = evalHist(1:iter);

switch stat
 case EXIT_ITERATIONS,
  info.statstr = 'Maximum number of iterations reached';
  
 case EXIT_OPTIMAL,
  info.statstr = 'Optimal solution found';
 
 case EXIT_FUNCTIONS,
  info.statstr = 'Maximum number of function evaluations reached';
end



function [f,g] = groupObjective(funObj,weight,idx,x,xgroup)

x(idx) = xgroup;
[f,g]  = funObj(x);
n      = norm(xgroup,2);
f      = f + weight * n;
g      = g(idx) + weight * xgroup / n;
