function [x,info] = epsL1(funObj,lambda,groups,weights,opts)
% EPSL1   Solver
%
%   [X,INFO] = EPSL1(FUNOBJ,LAMBDA,GROUPS,WEIGHTS,OPTS) solves
%
%       Minimize  funObj(x) + lambda * ||x||_WG,1
%   or
%       Minimize  funObj(x) + lambda * sum(w_i ||x_Gi||_2)
%
%   With weighted group norm depending on GROUPS and WEIGHTS.
%   The GROUPS vector indicates for each entry of variable X
%   which group (number) it belongs to. The length of WEIGHTS 
%   should correspond to the number of groups used. Group
%   numbering should be done from 1 to the number of groups.
%
%   The following fields of the OPTS options structure are
%   recognized:
%
%   epsl1tol    Tolerance [1e-8]
%   epsilon	Smoothing parameter [5e-10]
%   maxiter     Maximum number of block iterations
%   maxfunc     Maximum number of function evaluations
%   verbosity   0 = no output, 1 = all output

if nargin < 5, opts = struct(); end;

if ~isfield(opts,'epsilon'  ), opts.epsilon   = 5e-10;           end;
if ~isfield(opts,'epsl1tol' ), opts.epsl1tol  = 1e-8;            end;
if ~isfield(opts,'maxiter'  ), opts.maxiter   = 1000;            end;
if ~isfield(opts,'maxfunc'  ), opts.maxfunc   = 10*opts.maxiter; end;
if ~isfield(opts,'verbosity'), opts.verbosity = 1;               end;


% Maximum value of gradient for non-zero blocks
n                   = length(groups);
tol                 = opts.epsl1tol
epsilon             = opts.epsilon;
options.TolFun      = tol;
options.TolX        = tol;
options.MaxFunEvals = opts.maxfunc;
options.MaxIter     = opts.maxiter;
options.LS_init     = 3;

if opts.verbosity == 0
   options.display = 'off';
end

global fHist
global normHist
global evalHist
fHist    = zeros(1,0);
normHist = zeros(1,0);
evalHist = zeros(1,0);

[x,fval,exitflag,output] = ...
    minFunc(@smoothObj,zeros(n,1),options,epsilon,groups,lambda,weights,funObj);

info.iter     = output.funcCount;
info.fHist    = fHist;
info.normHist = normHist;
info.evalHist = evalHist;

end

function [f,g] = smoothObj(x,epsilon,groups,lambda,weights,funObj)

[f,g] = funObj(x);

global fHist
global normHist
global evalHist

fHist(1,end+1) = f;
if size(evalHist,2) == 0
    evalHist = 1;
else
    evalHist(1,end+1) = evalHist(1,end)+1;
end

for i = 1:max(groups)
    idx = (groups == i);
    reg(i,1) = sqrt(sum(x(idx).^2) + epsilon);
    f = f + lambda*weights(i)*reg(i);
    g(idx) = g(idx) + lambda*weights(i)*x(idx)/reg(i);
end

normHist(1,end+1) = sum(reg.*weights);
end
